unit LPRat;
{
Solution linear programming task in fractional numbers
with absolute precision
for Delphi 3.0
Copyright (c) 1996-1997 Dmitriy Anisimkov
e-mail anisimkov@yahoo.com
phone +7 (381)-2217774
}
interface

uses MxEl//it must be an alias for unit Rational
,matrix,utils,classes;

type

pVariable = ^tVariable;
tVariable = record
  TypeLimit : (Positive,Any, Binom);
  HiLimit : TElement;
  Value : TElement; // 1..Max in array for trying unbroken values 0 for leader
end;

tLPMatrix =class;
pArrayOfVariable = ^tArrayOfVariable;
tArrayOfVariable = array[0..MaxInt div sizeof(tVariable)-1] of tVariable;
pArrayOfComp = ^tArrayOfComp;
tArrayOfComp = array [0..MaxInt div sizeof(comp)-1] of comp;
tLPMatrixEvent = procedure(m : TLPMatrix) of object;

tLPMatrix =class(TMatrix)
private
  fVarWorkCount : integer;
  fVarRealCount : integer;
  FVariables : pArrayOfVariable;
  FVarIdx : pArrayOfInteger;
  FAnyExists : boolean;
  function GetC(j : integer) : tElement;
  procedure SetC(j : integer; it : tElement);
  procedure ModJord(r,s : integer);
  procedure MakeLower(s : integer);
  function NegativeC : integer;
  function NegativeB : integer;
  function InColumn(s : integer) : integer;
  function InRow(r : integer) : integer;
  function In_Row(r : integer) : integer;
  function NotNullInCol(s : integer) : integer;
  function NotNullInRow(r : integer) : integer;
  function IsEqual(r : integer) : boolean;
  function GetX(i : integer) : tElement;
  procedure AllocVars;
  function GetVariable(i : integer) : pVariable;
  function GetVarIdx(i : integer) : integer;
  procedure SetVarIdx(i,vi : integer);
  function FindAny(r : integer) : integer;
protected
  procedure ExchangeRows(i1,i2 : integer); override;
public
{    .      
     }
  CopiedFrom : TLPMatrix;
  OnChangeLeader : tLPMatrixEvent;
  OnSimplex : tLPMatrixEvent;
  OnSkip : tLPMatrixEvent;
  OnSkipNode : tLPMatrixEvent;
  OnString : tLPMatrixEvent;
  SolutionExists : boolean;
  property AnyExists : boolean write FAnyExists;
  property Variables[i : integer] : pVariable read GetVariable;
  property VarIdx[i : integer] : Integer read GetVarIdx write SetVarIdx;
  constructor Create(aMaxI,aMaxJ,Vars : integer);
  constructor CreateEx(aMaxI,aMaxJ,Vars,VarIdxCount : integer); //need set variables
  constructor Copy(from : TMatrix); override;
  destructor Destroy; override;
  procedure Solution;
  procedure ModifSimplexMethod;
  procedure UnbrokenSolution;
  procedure GetSolution(var toar : array of Double);
  property b[i : integer] : tElement index 0 read GetElement write SetElement;
  property c[i : integer] : tElement read GetC write SetC;
  property VarCount : integer read fVarRealCount; // Number of variables
  property MaxVar : integer read fVarWorkCount;// Number of variables in current task
  property X[i : integer] : tElement read GetX;
end;

implementation

uses sysutils,LPMatxF;

type
tNoSolution = class(Exception)
  constructor Create;
end;

constructor tNoSolution.Create;
begin
  inherited Create('No solution');
end;

procedure tLPMatrix.AllocVars;
begin
  if (fVarWorkCount>MaxI+MaxJ) then
    raise Exception.Create('Vars>MaxI+MaxJ');
  {$Q+}
  FVariables := AllocMem((fVarRealCount+1)*sizeof(FVariables[0]));
  FVarIdx := AllocMem((fVarWorkCount+1)*sizeof(VarIdx[0]));
  {$Q-}
  Variables[0].Value := NegativeUnlimited;
end;

constructor tLPMatrix.Create(aMaxI,aMaxJ,Vars : integer);
begin
  CreateEx(aMaxI,aMaxJ,Vars,Vars);
  for Vars := 0 to MaxVar do begin
    Variables[Vars].Value := NegativeUnlimited;
    VarIdx[Vars]:=Vars;
  end;
end;

constructor tLPMatrix.CreateEx(aMaxI,aMaxJ,Vars,VarIdxCount : integer);
begin
  inherited Create(aMaxI+1,aMaxJ+1);
  fVarRealCount := Vars;
  fVarWorkCount := VarIdxCount;
  AllocVars;
end;

constructor tLPMatrix.Copy(from : TMatrix);
begin
  inherited;
  if from is tLPMatrix then begin
    if Assigned(tLPMatrix(From).CopiedFrom) then
      CopiedFrom := tLPMatrix(From).CopiedFrom
    else CopiedFrom := tLPMatrix(From);
    OnChangeLeader := CopiedFrom.OnChangeLeader;
    OnSimplex := CopiedFrom.OnSimplex;
    OnSkipNode := CopiedFrom.OnSkipNode;
    fVarRealCount := tLPMatrix(from).fVarRealCount;
    fVarWorkCount := tLPMatrix(from).fVarWorkCount;
    AllocVars;
    move(tLPMatrix(from).FVariables^,FVariables^,(fVarRealCount+1)*sizeof(tVariable));
    move(tLPMatrix(from).FVarIdx^,FVarIdx^,(fVarWorkCount+1)*sizeof(VarIdx[0]));
  end;
end;

destructor tLPMatrix.Destroy;
begin
  FreeMem(FVariables);
  FreeMem(FVarIdx);
  inherited Destroy;
end;

procedure tLPMatrix.ExchangeRows(i1,i2 : integer);
begin
  inherited;
  ExchangeMem(FVarIdx[i1+MaxJ],FVarIdx[i2+MaxJ],Sizeof(VarIdx[0]));
end;

function tLPMatrix.GetVariable(i : integer) : pVariable;
begin
  if (i<0) or (i>fVarRealCount) then
    raise ERangeError.Create('in LPMatrix');
  result := @FVariables[i];
end;

function tLPMatrix.GetVarIdx(i : integer) : integer;
begin
  if (i<0) or (i>MaxVar) then
    raise ERangeError.Create('SetVarIdx in LPMatrix');
  result := FVarIdx[i];
end;

procedure tLPMatrix.SetVarIdx(i,vi : integer);
begin
  if (i<0) or (i>MaxVar) then
    raise ERangeError.Create('GetVarIdx in LPMatrix');
  FVarIdx[i] := vi;
end;

procedure tLPMatrix.GetSolution(var toar : array of Double);
var
  cn : integer;
begin
  for cn := 0 to high(toar) do toar[cn] := ToDouble(X[cn+1]);
end;

function tLPMatrix.NotNullInRow(r : integer) : integer;
var
  j : integer;
begin
  result := 0;
  for J :=  MaxJ downto 1 do begin
    if A[r,J].numer<>0 then begin
      result := J;
      break;
    end;
  end;
end;

function tLPMatrix.NotNullInCol(s : integer) : integer;
begin
  for result := 1 to MaxI do
    if A[result,s].numer<>0 then
      exit;
  result := 0;
end;

function tLPMatrix.FindAny(r : integer) : integer;
var
  j : integer;
  aa : boolean;
begin
  result := 0;
  if not FAnyExists then exit;
  AA := false;
  for J :=  1 to MaxJ do begin
    if (Variables[VarIdx[J]].TypeLimit=Any) then begin
      aa := true;
      if (r=0) or (A[r,J].numer<>0) then begin
        result := J;
        break;
      end;
    end;
  end;
  FAnyExists := aa;
end;

procedure tLPMatrix.Solution;
var
  nxr,nxs : integer;
begin
  SolutionExists := false;
  while MaxVar<MaxI+MaxJ do begin // Normalisation
    nxr := MaxVar-MaxJ+1;
    nxs := FindAny(nxr);
    if nxs=0 then begin
      if B[nxr].numer<0 then nxs := InRow(nxr) else
      nxs := In_Row(nxr);
    end;
    if nxs=0 then exit;
    ModJord(nxr,nxs);
  end;
  while true do begin
    nxs := FindAny(0);
    if nxs=0 then break;
    nxr := NotNullInCol(nxs);
    if nxr=0 then begin
      ErrMsg('Unlimited solution');
      exit;
    end;
    ModJord(nxr,nxs);
  end;
  repeat // Base Solution
     nxr := NegativeB;
     if nxr=0 then break;
     nxs := InRow(nxr);
     if nxs=0 then exit;
     ModJord(nxr,nxs);
  until false;
  repeat // Optimal solution
    nxs := NegativeC;
    if nxs=0 then break;
    nxr := InColumn(nxs);
    if nxr=0 then begin
       ErrMsg('Unlimited solution');
       exit;
    end;
    ModJord(nxr,nxs);
  until false;
  SolutionExists := true;
end;

procedure tLPMatrix.ModifSimplexMethod;
var
  IB : pArrayOfElements;
  Pi : pArrayOfElements;
  i,j : integer;
begin
  IB := AllocMem(sizeof(IB[0])*RowCount*RowCount);
  Pi := IB;
  for i := 0 to RowCount-1 do for j := 0 to RowCount-1 do
    if i=j then ib[idx(i,j)] := ONE
    else ib[idx(i,j)] := NULL;
  FreeMem(IB);
end;

procedure tLPMatrix.UnbrokenSolution;
var
  i,j : integer;
  m : array [boolean] of TLPMatrix;
  cb,best,c : boolean;
  Bi : TElement;
begin
  for i:=MaxI downto 1 do begin
    Bi := B[i];  //find broken result in binom limit
    if (Variables[VarIdx[i+MaxJ]].TypeLimit=Binom)
    and not CompareMem(@Bi,@One,sizeof(Bi))
    and not CompareMem(@Bi,@Null,sizeof(Bi))
    then begin
      if MaxVar=MaxI then begin // no unbroken solution
        Variables[0].Value := NegativeUnlimited;
        if Assigned(OnSkipNode) then OnSkipNode(self);
        exit;
      end;
    try
      m[false] := TLPMatrix.Copy(Self); //ramification
      m[false].ExchangeRows(i,MaxI);
      Dec(m[false].fVarWorkCount);
      m[false].Variables[VarIdx[i+MaxJ]].Value := NULL;
      m[true] := TLPMatrix.Copy(m[false]);
      m[true].Variables[VarIdx[i+MaxJ]].Value := ONE;
      m[true].B[MaxI] := SumRC(m[true].B[MaxI],-1);
      if m[true].B[MaxI].numer<0 then
        for j:=0 to MaxJ do m[true][MaxI,j]:=Negative(m[true][MaxI,j]);
      m[false].Solution;
      m[true].Solution;
      if Assigned(OnSimplex) then OnSimplex(Self);
      best := Compare(m[false][0,0],m[true][0,0])<0; // who is better
      for cb:=false to true do begin
        c := best xor cb;
        if not m[c].SolutionExists then continue;
        if compare(m[c][0,0],Variables[0].Value)<0 then begin
          if Assigned(OnSkipNode) then OnSkipNode(m[c]);
          exit;
        end;
        m[c].Variables[0].Value := Variables[0].Value;
        m[c].UnbrokenSolution;
        if Compare(m[c].Variables[0].Value,Variables[0].Value)>0 then //Change leader
           move(m[c].FVariables^,FVariables^,sizeof(tVariable)*VarCount);
      end;
      exit;
    finally
      m[false].free;
      m[true].free;
    end;
    end;
  end;
  if Assigned(OnSkipNode) then OnSkipNode(Self);
  if Compare(A[0,0],Variables[0].Value)>0 then begin
    Variables[0].Value := A[0,0];
    for i:=1 to MaxJ do Variables[VarIdx[i]].Value := null;
    for i:=MaxJ+1 to MaxVar do
      Variables[VarIdx[i]].Value := B[i-MaxJ];
    if Assigned(OnChangeLeader) then OnChangeLeader(Self);
  end;
end;


function tLPMatrix.GetX(i : integer) : tElement;
var
  cv : integer;
begin
  for cv := 1 to MaxVar do begin
    if VarIdx[cv]=i then begin
      if cv>MaxJ then result := B[cv-MaxJ]
      else result:=NULL;
      Exit;
    end;
  end;
  raise Exception.Create('Error var ordering');
end;

function tLPMatrix.NegativeC : integer;
var
  min : tElement;
  j : integer;
begin
  min := NULL;
  result := 0;
  for j := 1 to MaxJ do
    if Compare(C[j],min)<0 then begin
      min := c[j];
      result := j;
  end;
end;

function tLPMatrix.NegativeB : integer;
var
  min : tElement;
  i : integer;
begin
  min := NULL;
  result := 0;
  for i := 1 to MaxI do if (Compare(B[i],min)<0)
   and (Variables[VarIdx[MaxJ+i]].TypeLimit<>Any) then begin
    min := b[i];
    result := i;
  end;
end;

// if MaxJ = VarCount then All equals;
// if MaxJ+MaxI = VarCount then normal form;
function tLPMatrix.IsEqual(r : integer) : boolean;
begin
  result := MaxJ + R > MaxVar;
end;

function tLPMatrix.GetC(j : integer) : tElement;
begin
  result := Datas^[idx(0,j)];
end;

procedure tLPMatrix.SetC(j : integer; it : tElement);
begin
  Datas^[idx(0,j)] := it;
end;

function tLPMatrix.InRow(r : integer) : integer;
var
  j : integer;
  MaxVal,Arj : tElement;
  CjDivArj : tElement;
begin
  MaxVal := NegativeUnlimited;
  result := 0;
  for j := 1 to MaxJ do begin
    Arj := A[r,j];
    if (Arj.numer<0) then begin
      CjDivArj := mxel.Multiply(C[j],MxEl.Inverse(Arj));
      if Compare(MaxVal,CjDivArj)<0 then begin
        MaxVal := CjDivArj;
        result := j;
        if CjDivArj.numer=0 then exit;
      end;
    end;
  end;
end;

function tLPMatrix.In_Row(r : integer) : integer;
var
  j : integer;
  BestVal,Arj : tElement;
  CjDivArj : tElement;
begin
  BestVal := Unlimited;
  result := 0;
  for j := 1 to MaxJ do begin
    Arj := A[r,j];
    if (Arj.numer>0) then begin
      CjDivArj := mxel.Multiply(C[j],MxEl.Inverse(Arj));
      if Compare(BestVal,CjDivArj)>0 then begin
        BestVal := CjDivArj;
        result := j;
      end;
    end;
  end;
end;

function tLPMatrix.InColumn(s : integer) : integer;
var
  minVal : tElement;
  i : integer;
  Ais,BiDivAis : tElement;
begin
  MinVal := unlimited;
  result := 0;
  for i := MaxI downto 1 do begin
    Ais := a[i,s];
    if (Ais.Numer>0) and (Variables[VarIdx[MaxJ+I]].TypeLimit<>Any) then begin
      BiDivAis := mxel.Multiply(B[i],MxEl.Inverse(Ais));
      if Compare(BiDivAis,MinVal)<0 then begin
        MinVal := BiDivAis;
        result := i;
        if BiDivAis.numer=0 then Exit;
      end;
    end;
  end;
end;

procedure tLPMatrix.MakeLower(s : integer);
var
  VarNum : integer;
begin
  VarNum := VarIdx[s];
  VarIdx[s] := VarIdx[MaxJ];
  Move(FVarIdx[MaxJ+1],FVarIdx[MaxJ],(MaxVar-MaxJ)*sizeof(VarIdx[0]));
  VarIdx[MaxVar] := VarNum;
  SetCols(ColCount-1);
end;

procedure tLPMatrix.ModJord(r,s : integer);
var
  i,j,rn : integer;
  NegInvArs,
  InvArs,NegArjDivArs : tElement;
  Aij,Dij : pElement;
  Ais : pArrayOfElements;
  Ranges : array [0..2] of record
    a : array[boolean] of integer;
    fa,fd : pElement;
  end;
type
pElementRec = ^tElementRec;
tElementRec = record
  e : tElement;
end;

begin
  InvArs := MxEl.Inverse(A[r,s]);
  NegInvArs := Negative(InvArs);
  Ais := pArrayOfElements(@Datas^[idx(0,s)]);
  with Ranges[0] do begin
    a[false] := 0;  a[true] := s-1;
    fa := @Datas^[0];
    fd := @Datas^[0];
  end;
  with Ranges[1] do begin
    a[false] := s+1;  a[true] := MaxJ-1;
    fa := @Datas^[idx(0,s+1)];
    fd := fa;
  end;
  with Ranges[2] do begin
    a[false] := MaxJ; a[true] := MaxJ;
    fa := @Datas^[idx(0,MaxJ)];
    if MaxVar=MaxI+MaxJ then begin
      fd := fa;
      if MaxJ=s then a[false] := MaxJ+7;//Make interval incorrect
    end else
      fd := @Datas^[idx(0,s)]
  end;
  for rn := Low(Ranges) to High(Ranges) do begin
    Aij := Ranges[rn].fa;
    Dij := Ranges[rn].fd;
  for j := Ranges[rn].a[false] to Ranges[rn].a[true] do begin
    NegArjDivArs := mxel.Multiply(Datas^[idx(r,j)],NegInvArs);
    for i := 0 to MaxI do begin
        if i<>r then
           Dij^ := Sum(Aij^,mxel.Multiply(Ais^[i],NegArjDivArs))
        else
           Dij^ := mxel.Multiply(Aij^,InvArs);
       inc(Aij);
       inc(Dij);
    end;
    if Assigned(OnString) then OnString(Self);
  end;
  end;
  if MaxVar=MaxI+MaxJ then begin
    for i:=0  to MaxI do with tElementRec(Ais^[i]) do
      e := mxel.Multiply(e,NegInvArs);
    A[r,s] := InvArs;
    ExchangeMem(FVarIdx[s],FVarIdx[MaxJ+r],Sizeof(VarIdx[0]));
  end else MakeLower(s);
  if Assigned(OnSkip) then OnSkip(self);
end;

end.

//*********************************************************************
procedure tLPMatrix.CheckUnbroken;
var
  i,j : integer;
  sigma,Mul4 : tElement;
  MayStop : boolean;
begin
  repeat
  i := 1;
  MayStop := true;
  while i <= MaxI do begin
  if (Variables[i+MaxJ].unbroken) and (B[i].denom<>1) then begin
    MayStop := false;
    AddRow;
    AddVariable;
    Mul4 := Euclide(B[i].numer,b[i].denom,true);
    sigma := PositiveFract(B[i]);
    sigma := mxel.Multiply(sigma,MxEl.inverse(negative(sumrc(sigma,-1))));
    B[MaxI] := negative(sigma);
    for j := 1 to MaxJ do
//      Datas[idx(MaxI,J)] :=  Negative(PositiveFract(MulRC(A[i,j],Mul4.denom)));
      if A[i,j].numer<0 then
      Datas[idx(MaxI,J)] := mxel.Multiply(A[i,j],Sigma)
      else Datas[idx(MaxI,J)]:=MulRC(A[i,j],-1);
    repeat
      NegativeRS;
      if nxr=0 then break;
      ModJord(nxr,nxs);
    until false;
  end;
  inc(i);
  end;
  until MayStop;
end;


